﻿//-----------------------------------------------------------------------------
// <copyright file="ComPortDevice.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2012 eQ-3 Entwicklung GmbH
// </copyright>
//-----------------------------------------------------------------------------
namespace RgbCubeDemoApp
{
    using System.Collections.Generic;
    using System.IO.Ports;

    /// <summary>
    /// Represents a wrapper for a device on a Com-Port with methods for sending and receiving commands.
    /// </summary>
    public class ComPortDevice
    {
        /// <summary>
        /// Constant for byte checking in method EnqSpecialBytes()
        /// </summary>
        private const byte STX = 0x02;

        /// <summary>
        /// Constant for byte checking in method EnqSpecialBytes()
        /// </summary>
        private const byte ENQ = 0x10;

        /// <summary>
        /// SerialPort of the wrapper
        /// </summary>
        private SerialPort serialPort;

        /// <summary>
        /// Initializes a new instance of the ComPortDevice class
        /// </summary>
        /// <param name="serialPort">Com-Port of the wrapper</param>
        public ComPortDevice(SerialPort serialPort)
        {
            this.serialPort = serialPort;
        }

        /// <summary>
        /// Writes a command to the device
        /// </summary>
        /// <param name="opcode">Command opcode</param>
        /// <param name="para">Command parameters</param>
        /// <param name="amountToRead">Amount of bytes which are read for the answer</param>
        /// <returns>Answer of the device</returns>
        public byte[] SendCommandToDevice(byte opcode, byte[] para, int amountToRead)
        {
            List<byte> buffer = new List<byte>();
            List<byte> dataFrame = new List<byte>();

            dataFrame.Add(opcode);
            dataFrame.AddRange(para);

            byte[] header = new byte[2];
            header[0] = (byte)((dataFrame.ToArray().Length >> 8) & 0xff);
            header[1] = (byte)(dataFrame.ToArray().Length & 0xff);
            header = this.EnqSpecialBytes(header); // Escape special chars from the header
            dataFrame = new List<byte>(this.EnqSpecialBytes(dataFrame.ToArray())); // Escape special chars from the dataframe

            // Calculate CRC16 for the startchar, the header and the dataframe
            CRC16 crc = new CRC16();
            crc.Shift(STX);
            crc.Shift(header);
            crc.Shift(dataFrame.ToArray());

            // Combine all data parts for transfering to the device
            List<byte> ouput = new List<byte>();
            ouput.Add(STX);
            ouput.AddRange(header);
            ouput.AddRange(dataFrame);
            ouput.AddRange(this.EnqSpecialBytes(crc.CrcBytes)); // Escape special chars from the crc

            int bytesWriteRequest = ouput.Count;
            this.serialPort.Write(ouput.ToArray(), 0, bytesWriteRequest);

            byte[] readData = this.ReadAnswerAfterCommand(opcode, amountToRead);
            return readData;
        }

        /// <summary>
        /// Gets the Answer of a transmitted command from the device
        /// </summary>
        /// <param name="expectedOpcode">Opcode of the transmitted command (needed for verification)</param>
        /// <param name="amountToRead">Amount of bytes which are read for the answer</param>
        /// <returns>Answer of the transmitted command from the device</returns>
        private byte[] ReadAnswerAfterCommand(byte expectedOpcode, int amountToRead)
        {
            /*
             * Aufbau der ausgelesenen Daten
             * 1 Byte: Startzeichen (STX)
             * 2 Byte: Länge der folgenden Daten
             * siehe Länge: ausgelesene Daten mit Opcode
             * 2 Byte: CRC16
             */
            const int READ_OFFSET = 5;
            byte[] readData = new byte[amountToRead + READ_OFFSET];
            this.serialPort.Read(readData, 0, amountToRead + READ_OFFSET);

            if (readData[0] == 0x02)
            {
                readData = this.DeenqSpecialBytes(readData);
                byte readOpcode = readData[2];
                if (readOpcode != expectedOpcode)
                {
                    return new byte[0];
                }

                return readData;
            }
            else
            {
                return new byte[0];
            }
        }

        /// <summary>
        /// Converts special bytes before transmitting the command
        /// </summary>
        /// <param name="buffer">command for the device</param>
        /// <returns>converted command</returns>
        private byte[] EnqSpecialBytes(byte[] buffer)
        {
            List<byte> temp = new List<byte>();

            for (int i = 0; i < buffer.Length; i++)
            {
                if (buffer[i] == STX)
                {
                    temp.Add(ENQ);
                    temp.Add(STX + 0x80);
                }
                else if (buffer[i] == ENQ)
                {
                    temp.Add(ENQ);
                    temp.Add(ENQ + 0x80);
                }
                else
                {
                    temp.Add(buffer[i]);
                }
            }

            return temp.ToArray();
        }

        /// <summary>
        /// Removes escape chars from the received data
        /// </summary>
        /// <param name="readData">Readed data from device</param>
        /// <returns>Data from command</returns>
        private byte[] DeenqSpecialBytes(byte[] readData)
        {
            List<byte> temp = new List<byte>();
            bool escaped = false;

            for (int i = 0; i < readData.Length; i++)
            {
                if (readData[i] == STX)
                {
                }
                else if (readData[i] == ENQ)
                {
                    escaped = true;
                }
                else if(escaped)
                {
                    temp.Add((byte)(readData[i] & 0x7F));
                    escaped = false;
                }
                else
                {
                    temp.Add(readData[i]);
                }
            }

            return temp.ToArray();
        }
    }
}