﻿//-----------------------------------------------------------------------------
// <copyright file="LedBingo.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2012 eQ-3 Entwicklung GmbH
// </copyright>
//-----------------------------------------------------------------------------
namespace RgbCubeDemoApp
{       
    using System;
    using System.Timers;

    /// <summary>
    /// Wrapper class for the LED-Bingo
    /// </summary>
    public class LedBingo
    {
        /// <summary>
        /// Device for the LED-Bingo
        /// </summary>
        private ComPortDevice device;

        /// <summary>
        /// Timer for showing the next Image
        /// </summary>
        private Timer viewTimer;

        /// <summary>
        /// Matrix of LEDs
        /// </summary>
        private LedCoordinate[] leds;

        /// <summary>
        /// Maximal possible Bingos
        /// </summary>
        private int totalBingos;

        /// <summary>
        /// Matrix of LEDs (indicates current state of each LED)
        /// </summary>
        private int[,,] ledsOn;

        /// <summary>
        /// Amount of turned on LEDs.
        /// </summary>
        private int ledOnCounter = 0;

        /// <summary>
        /// Amount of Bingos.
        /// </summary>
        private int isBingo = 0;

        /// <summary>
        /// Maximal amount of usable LEDs.
        /// </summary>
        private int maxLeds = 0;

        /// <summary>
        /// Initializes a new instance of the LedBingo class
        /// </summary>
        /// <param name="device">Comport device.</param>
        /// <param name="totalBingos">Maximal possible Bingos</param>
        /// <param name="maxLeds">Maximal amount of usable LEDs.</param>
        public LedBingo(ComPortDevice device, int totalBingos, int maxLeds)
        {
            this.device = device;
            this.totalBingos = totalBingos;
            this.maxLeds = maxLeds;
            this.ledsOn = new int[5, 5, 5];
            Random r = new Random();
            int ledCount = 0;
            this.leds = new LedCoordinate[125];

            for (int x = 0; x < 5; x++)
            {
                for (int y = 0; y < 5; y++)
                {
                    for (int z = 0; z < 5; z++)
                    {
                        LedCoordinate newCoordinate = new LedCoordinate(x, y, z, 255, 255, 255);
                        this.leds[ledCount] = newCoordinate;
                        ledCount++;
                    }
                }
            }

            // Mixes the position of the LEDs
            for (int j = 0; j < 1000; j++)
            {
                int first = r.Next(125);
                int second = r.Next(125);
                LedCoordinate temp = this.leds[first];
                this.leds[first] = this.leds[second];
                this.leds[second] = temp;
            }

            this.viewTimer = new Timer(100.0);
            this.viewTimer.Elapsed += new ElapsedEventHandler(this.ViewTimer_Elapsed);
        }

        /// <summary>
        /// Starts selecting the random LEDs 
        /// </summary>
        public void Start()
        {
            // Switches off all LEDs
            this.device.SendCommandToDevice(Convert.ToByte('G'), new byte[375], 10);

            this.viewTimer.Start();                 
        }

        /// <summary>
        /// Stops selecting the random LEDs
        /// </summary>
        public void Stop()
        {
            if (this.viewTimer.Enabled)
            {
                this.viewTimer.Stop();
            }
                
            this.ShowResult();  
        }

        /// <summary>
        /// Checks for Bingos 
        /// </summary>
        /// <param name="ledsOn">all turned on LEDs</param>
        /// <param name="ledCoordinate">coordinates of all turned on LEDs</param>
        /// <returns>bingo as amount of Bingos</returns>
        private static int CheckForBingo(int[,,] ledsOn, LedCoordinate ledCoordinate)
        {
            int bingo = 0;

            // Checks for a Bingo on the X-axis
            for (int x = 0; x < 5; x++)
            {
                if (ledsOn[x, ledCoordinate.Y, ledCoordinate.Z] == 0)
                {
                    break;
                }
                else if (x == 4)
                {
                    ledsOn[0, ledCoordinate.Y, ledCoordinate.Z] = 2;
                    ledsOn[1, ledCoordinate.Y, ledCoordinate.Z] = 2;
                    ledsOn[2, ledCoordinate.Y, ledCoordinate.Z] = 2;
                    ledsOn[3, ledCoordinate.Y, ledCoordinate.Z] = 2;
                    ledsOn[4, ledCoordinate.Y, ledCoordinate.Z] = 2;
                    bingo++;

                    // Checks the coordinates of the Bingo
                    // Console.WriteLine("Die Koordinaten des Bingos auf der X-Achse sind: x= {0} y=: {1} z=: {2}",ledCoordinate.X, ledCoordinate.Y, ledCoordinate.Z);
                }
            }

            // Checks for a Bingo on the Y-axis
            for (int y = 0; y < 5; y++)
            {
                if (ledsOn[ledCoordinate.X, y, ledCoordinate.Z] == 0)
                {
                    break;
                }
                else if (y == 4)
                {
                    ledsOn[ledCoordinate.X, 0, ledCoordinate.Z] = 2;
                    ledsOn[ledCoordinate.X, 1, ledCoordinate.Z] = 2;
                    ledsOn[ledCoordinate.X, 2, ledCoordinate.Z] = 2;
                    ledsOn[ledCoordinate.X, 3, ledCoordinate.Z] = 2;
                    ledsOn[ledCoordinate.X, 4, ledCoordinate.Z] = 2;
                    bingo++;

                    // Checks the coordinates of the Bingo
                    // Console.WriteLine("Koordinaten Bingos Y-Achse x: {0} y: {1} z: {2}", ledCoordinate.X, ledCoordinate.Y, ledCoordinate.Z);
                }
            }

            // Checks for a Bingo on the Z-axis
            for (int z = 0; z < 5; z++)
            {
                if (ledsOn[ledCoordinate.X, ledCoordinate.Y, z] == 0)
                {
                    break;
                }
                else if (z == 4)
                {
                    ledsOn[ledCoordinate.X, ledCoordinate.Y, 0] = 2;
                    ledsOn[ledCoordinate.X, ledCoordinate.Y, 1] = 2;
                    ledsOn[ledCoordinate.X, ledCoordinate.Y, 2] = 2;
                    ledsOn[ledCoordinate.X, ledCoordinate.Y, 3] = 2;
                    ledsOn[ledCoordinate.X, ledCoordinate.Y, 4] = 2;
                    bingo++;

                    // Checks the coordinates of the Bingo
                    // Console.WriteLine("Koordinaten Bingos Z-Achse x: {0} y: {1} z: {2}", ledCoordinate.X, ledCoordinate.Y, ledCoordinate.Z);
                }
            }

            return bingo;
        }

        /// <summary>
        /// Checks for a diagonal Bingo
        /// </summary>
        /// <param name="ledsOn">all turned on LEDs</param>
        /// <param name="ledCoordinate">coordinates of all turned on LEDs</param>
        /// <returns>bingo as amount of Bingos</returns>
        private static int CheckForDiagonalBingo(int[,,] ledsOn, LedCoordinate ledCoordinate)
        {
            int bingo = 0;

            // Checks for a diagonal Bingo on the X-axis
            if (ledCoordinate.Y == ledCoordinate.Z)
            {
                if (ledsOn[ledCoordinate.X, 0, 0] > 0 && ledsOn[ledCoordinate.X, 1, 1] > 0 && ledsOn[ledCoordinate.X, 2, 2] > 0 && ledsOn[ledCoordinate.X, 3, 3] > 0 && ledsOn[ledCoordinate.X, 4, 4] > 0)
                {
                    ledsOn[ledCoordinate.X, 0, 0] = 2;
                    ledsOn[ledCoordinate.X, 1, 1] = 2;
                    ledsOn[ledCoordinate.X, 2, 2] = 2;
                    ledsOn[ledCoordinate.X, 3, 3] = 2;
                    ledsOn[ledCoordinate.X, 4, 4] = 2;
                    bingo++;

                    // Checks the coordinates of the Bingo
                    // Console.WriteLine("Koordinaten Bingos diagonal 00 X-Achse x: {0} y: {1} z: {2}", ledCoordinate.X, ledCoordinate.Y, ledCoordinate.Z);
                }
            }

            if (ledCoordinate.Y == 4 - ledCoordinate.Z)
            {
                if (ledsOn[ledCoordinate.X, 0, 4] > 0 && ledsOn[ledCoordinate.X, 1, 3] > 0 && ledsOn[ledCoordinate.X, 2, 2] > 0 && ledsOn[ledCoordinate.X, 3, 1] > 0 && ledsOn[ledCoordinate.X, 4, 0] > 0)
                {
                    ledsOn[ledCoordinate.X, 0, 4] = 2;
                    ledsOn[ledCoordinate.X, 1, 3] = 2;
                    ledsOn[ledCoordinate.X, 2, 2] = 2;
                    ledsOn[ledCoordinate.X, 3, 1] = 2;
                    ledsOn[ledCoordinate.X, 4, 0] = 2;
                    bingo++;

                    // Checks the coordinates of the Bingo
                    // Console.WriteLine("Koordinaten Bingos diagonal 04 X-Achse x: {0} y: {1} z: {2}", ledCoordinate.X, ledCoordinate.Y, ledCoordinate.Z);
                }
            }

            // Checks for a diagonal Bingo on the Y-axis
            if (ledCoordinate.X == ledCoordinate.Z)
            {
                if (ledsOn[0, ledCoordinate.Y, 0] > 0 && ledsOn[1, ledCoordinate.Y, 1] > 0 && ledsOn[2, ledCoordinate.Y, 2] > 0 && ledsOn[3, ledCoordinate.Y, 3] > 0 && ledsOn[4, ledCoordinate.Y, 4] > 0)
                {
                    ledsOn[0, ledCoordinate.Y, 0] = 2;
                    ledsOn[1, ledCoordinate.Y, 1] = 2;
                    ledsOn[2, ledCoordinate.Y, 2] = 2;
                    ledsOn[3, ledCoordinate.Y, 3] = 2;
                    ledsOn[4, ledCoordinate.Y, 4] = 2;
                    bingo++;

                    // Checks the coordinates of the Bingo
                    // Console.WriteLine("Koordinaten Bingos diagonal 00 Y-Achse x: {0} y: {1} z: {2}", ledCoordinate.X, ledCoordinate.Y, ledCoordinate.Z);
                }
            }

            if (ledCoordinate.X == 4 - ledCoordinate.Z)
            {
                if (ledsOn[0, ledCoordinate.Y, 4] > 0 && ledsOn[1, ledCoordinate.Y, 3] > 0 && ledsOn[2, ledCoordinate.Y, 2] > 0 && ledsOn[3, ledCoordinate.Y, 1] > 0 && ledsOn[4, ledCoordinate.Y, 0] > 0)
                {
                    ledsOn[0, ledCoordinate.Y, 4] = 2;
                    ledsOn[1, ledCoordinate.Y, 3] = 2;
                    ledsOn[2, ledCoordinate.Y, 2] = 2;
                    ledsOn[3, ledCoordinate.Y, 1] = 2;
                    ledsOn[4, ledCoordinate.Y, 0] = 2;
                    bingo++;

                    // Checks the coordinates of the Bingo
                    // Console.WriteLine("Koordinaten Bingos diagonal 04 Y-Achse x: {0} y: {1} z: {2}", ledCoordinate.X, ledCoordinate.Y, ledCoordinate.Z);
                }
            }

            // Checks for a diagonal Bingo on the Z-axis
            if (ledCoordinate.X == ledCoordinate.Y)
            {
                if (ledsOn[0, 0, ledCoordinate.Z] > 0 && ledsOn[1, 1, ledCoordinate.Z] > 0 && ledsOn[2, 2, ledCoordinate.Z] > 0 && ledsOn[3, 3, ledCoordinate.Z] > 0 && ledsOn[4, 4, ledCoordinate.Z] > 0)
                {
                    ledsOn[0, 0, ledCoordinate.Z] = 2;
                    ledsOn[1, 1, ledCoordinate.Z] = 2;
                    ledsOn[2, 2, ledCoordinate.Z] = 2;
                    ledsOn[3, 3, ledCoordinate.Z] = 2;
                    ledsOn[4, 4, ledCoordinate.Z] = 2;
                    bingo++;

                    // Checks the coordinates of the Bingo
                    // Console.WriteLine("Koordinaten Bingos diagonal 00 Z-Achse x: {0} y: {1} z: {2}", ledCoordinate.X, ledCoordinate.Y, ledCoordinate.Z);
                }
            }

            if (ledCoordinate.X == 4 - ledCoordinate.Y)
            {
                if (ledsOn[0, 4, ledCoordinate.Z] > 0 && ledsOn[1, 3, ledCoordinate.Z] > 0 && ledsOn[2, 2, ledCoordinate.Z] > 0 && ledsOn[3, 1, ledCoordinate.Z] > 0 && ledsOn[4, 0, ledCoordinate.Z] > 0)
                {
                    ledsOn[0, 4, ledCoordinate.Z] = 2;
                    ledsOn[1, 3, ledCoordinate.Z] = 2;
                    ledsOn[2, 2, ledCoordinate.Z] = 2;
                    ledsOn[3, 1, ledCoordinate.Z] = 2;
                    ledsOn[4, 0, ledCoordinate.Z] = 2;
                    bingo++;

                    // Checks the coordinates of the Bingo
                    // Console.WriteLine("Koordinaten Bingos diagonal 04 Z-Achse x: {0} y: {1} z: {2}", ledCoordinate.X, ledCoordinate.Y, ledCoordinate.Z);
                }
            }

            return bingo;
        }

        /// <summary>
        /// Checks for a diagonal Bingo through the middle 
        /// </summary>
        /// <param name="ledsOn">all turned on LEDs</param>
        /// <param name="ledCoordinate">coordinates of all turned on LEDs</param>
        /// <returns>bingo as amount of Bingos</returns>
        private static int CheckForCrossoverDiagonalBingo(int[,,] ledsOn, LedCoordinate ledCoordinate)
        {
            int bingo = 0;
            if ((ledCoordinate.X == 0 && ledCoordinate.Y == 0 && ledCoordinate.Z == 0) || (ledCoordinate.X == 1 && ledCoordinate.Y == 1 && ledCoordinate.Z == 1) || (ledCoordinate.X == 2 && ledCoordinate.Y == 2 && ledCoordinate.Z == 2) || (ledCoordinate.X == 3 && ledCoordinate.Y == 3 && ledCoordinate.Z == 3) || (ledCoordinate.X == 4 && ledCoordinate.Y == 4 && ledCoordinate.Z == 4))
            {
                if (ledsOn[0, 0, 0] > 0 && ledsOn[1, 1, 1] > 0 && ledsOn[2, 2, 2] > 0 && ledsOn[3, 3, 3] > 0 && ledsOn[4, 4, 4] > 0)
                {
                    ledsOn[0, 0, 0] = 2;
                    ledsOn[1, 1, 1] = 2;
                    ledsOn[2, 2, 2] = 2;
                    ledsOn[3, 3, 3] = 2;
                    ledsOn[4, 4, 4] = 2;
                    bingo++;

                    // Checks the coordinates of the Bingo
                    // Console.WriteLine("Koordinaten Bingos diagonal 000 Mitte x: {0} y: {1} z: {2}", ledCoordinate.X, ledCoordinate.Y, ledCoordinate.Z);
                }
            }

            if ((ledCoordinate.X == 4 && ledCoordinate.Y == 0 && ledCoordinate.Z == 0) || (ledCoordinate.X == 3 && ledCoordinate.Y == 1 && ledCoordinate.Z == 1) || (ledCoordinate.X == 2 && ledCoordinate.Y == 2 && ledCoordinate.Z == 2) || (ledCoordinate.X == 1 && ledCoordinate.Y == 3 && ledCoordinate.Z == 3) || (ledCoordinate.X == 0 && ledCoordinate.Y == 4 && ledCoordinate.Z == 4))
            {
                if (ledsOn[4, 0, 0] > 0 && ledsOn[3, 1, 1] > 0 && ledsOn[2, 2, 2] > 0 && ledsOn[1, 3, 3] > 0 && ledsOn[0, 4, 4] > 0)
                {
                    ledsOn[4, 0, 0] = 2;
                    ledsOn[3, 1, 1] = 2;
                    ledsOn[2, 2, 2] = 2;
                    ledsOn[1, 3, 3] = 2;
                    ledsOn[0, 4, 4] = 2;
                    bingo++;

                    // Checks the coordinates of the Bingo
                    // Console.WriteLine("Koordinaten Bingos diagonal 400 Mitte x: {0} y: {1} z: {2}", ledCoordinate.X, ledCoordinate.Y, ledCoordinate.Z);
                }
            }

            if ((ledCoordinate.X == 4 && ledCoordinate.Y == 0 && ledCoordinate.Z == 4) || (ledCoordinate.X == 3 && ledCoordinate.Y == 1 && ledCoordinate.Z == 3) || (ledCoordinate.X == 2 && ledCoordinate.Y == 2 && ledCoordinate.Z == 2) || (ledCoordinate.X == 1 && ledCoordinate.Y == 3 && ledCoordinate.Z == 1) || (ledCoordinate.X == 0 && ledCoordinate.Y == 4 && ledCoordinate.Z == 0))
            {
                if (ledsOn[4, 0, 4] > 0 && ledsOn[3, 1, 3] > 0 && ledsOn[2, 2, 2] > 0 && ledsOn[1, 3, 1] > 0 && ledsOn[0, 4, 0] > 0)
                {
                    ledsOn[4, 0, 4] = 2;
                    ledsOn[3, 1, 3] = 2;
                    ledsOn[2, 2, 2] = 2;
                    ledsOn[1, 3, 1] = 2;
                    ledsOn[0, 4, 0] = 2;
                    bingo++;

                    // Checks the coordinates of the Bingo
                    // Console.WriteLine("Koordinaten Bingos diagonal 404 Mitte x: {0} y: {1} z: {2}", ledCoordinate.X, ledCoordinate.Y, ledCoordinate.Z);
                }
            }

            if ((ledCoordinate.X == 0 && ledCoordinate.Y == 0 && ledCoordinate.Z == 4) || (ledCoordinate.X == 1 && ledCoordinate.Y == 1 && ledCoordinate.Z == 3) || (ledCoordinate.X == 2 && ledCoordinate.Y == 2 && ledCoordinate.Z == 2) || (ledCoordinate.X == 3 && ledCoordinate.Y == 3 && ledCoordinate.Z == 1) || (ledCoordinate.X == 4 && ledCoordinate.Y == 4 && ledCoordinate.Z == 0))
            {
                if (ledsOn[0, 0, 4] > 0 && ledsOn[1, 1, 3] > 0 && ledsOn[2, 2, 2] > 0 && ledsOn[3, 3, 1] > 0 && ledsOn[4, 4, 0] > 0)
                {
                    ledsOn[0, 0, 4] = 2;
                    ledsOn[1, 1, 3] = 2;
                    ledsOn[2, 2, 2] = 2;
                    ledsOn[3, 3, 1] = 2;
                    ledsOn[4, 4, 0] = 2;
                    bingo++;

                    // Checks the coordinates of the Bingo
                    // Console.WriteLine("Koordinaten Bingos diagonal 004 Mitte x: {0} y: {1} z: {2}", ledCoordinate.X, ledCoordinate.Y, ledCoordinate.Z);
                }
            }

            return bingo;
        }

        /// <summary>
        /// Shows the result and colors the LEDs
        /// </summary> 
        private void ShowResult()
        {
            byte[] cubeImageBytes = new byte[375];
            Console.WriteLine();
            Console.WriteLine("Anzahl der benötigten LEDs für {0} Bingos: {1}", this.isBingo, this.ledOnCounter);

            for (int x = 0; x < 5; x++)
            {
                for (int y = 0; y < 5; y++)
                {
                    for (int z = 0; z < 5; z++)
                    {
                        int target = (x * 75) + (y * 15) + (z * 3);

                        if (this.ledsOn[x, y, z] == 2)
                        {
                            // Colors the Bingo-LEDs in #ff0000 (red)
                            cubeImageBytes[target] = 0xff;
                            cubeImageBytes[target + 1] = 0x00;
                            cubeImageBytes[target + 2] = 0x00;
                        }
                        else if (this.ledsOn[x, y, z] == 1)
                        {
                            // Colors the random LEDs in #ffffff (white)
                            cubeImageBytes[target] = 0xff;
                            cubeImageBytes[target + 1] = 0xff;
                            cubeImageBytes[target + 2] = 0xff;
                        }
                        else
                        {
                            // Colors the unused LEDs in #000000 (black)
                            cubeImageBytes[target] = 0x00;
                            cubeImageBytes[target + 1] = 0x00;
                            cubeImageBytes[target + 2] = 0x00;
                        }
                    }
                }
            }

            this.device.SendCommandToDevice(Convert.ToByte('G'), cubeImageBytes, 10);
        }

        /// <summary>
        /// Sets the LEDs in the intervall
        /// </summary>
        /// <param name="sender">Sender object</param>
        /// <param name="e">Event arguments</param>
        private void ViewTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            if (this.ledOnCounter < this.maxLeds && this.isBingo < this.totalBingos)
            {
                LedCoordinate nextLed = this.leds[this.ledOnCounter];
                this.ledsOn[nextLed.X, nextLed.Y, nextLed.Z] = 1;
                byte[] ledByte = new byte[6];
                ledByte[0] = (byte)nextLed.X;
                ledByte[1] = (byte)nextLed.Y;
                ledByte[2] = (byte)nextLed.Z;
                ledByte[3] = (byte)nextLed.R;
                ledByte[4] = (byte)nextLed.G;
                ledByte[5] = (byte)nextLed.B;

                this.device.SendCommandToDevice(Convert.ToByte('S'), ledByte, 10);

                this.isBingo += LedBingo.CheckForBingo(this.ledsOn, nextLed);
                this.isBingo += LedBingo.CheckForDiagonalBingo(this.ledsOn, nextLed);
                this.isBingo += LedBingo.CheckForCrossoverDiagonalBingo(this.ledsOn, nextLed);

                this.ledOnCounter++;
            }
            else
            {
                this.ShowResult();
                this.viewTimer.Stop();
            }
        }  
    }
}
