//-----------------------------------------------------------------------------
// <copyright file="LedColorGradient.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2010 eQ-3 Entwicklung GmbH
// </copyright>
//-----------------------------------------------------------------------------

namespace RgbCubeDemoApp
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.Timers;

    /// <summary>
    /// Led Color Gradient class for showing a color gradient
    /// </summary>
    public class LedColorGradient
    {
        /// <summary>
        /// Amount of LEDs each side (x, y, z)
        /// </summary>
        public const int AmountOfLedsEachSide = 5;

        /// <summary>
        /// Amount of bytes for a complete image (5 * 5 * 5 * 3)
        /// </summary>
        public const int Capacity = 375;

        /// <summary>
        /// Device for the color gradient
        /// </summary>
        private ComPortDevice device;

        /// <summary>
        /// Timer for showing the next Image
        /// </summary>
        private Timer viewTimer;

        /// <summary>
        /// List of the recently used colors
        /// </summary>
        private List<Color> recentlyUsedColors;

        /// <summary>
        /// Current hue value
        /// </summary>
        private double hueValue;

        /// <summary>
        /// Initializes a new instance of the LedColorGradient class
        /// </summary>
        /// <param name="device">Device for the colour gradient</param>
        public LedColorGradient(ComPortDevice device)
        {
            this.device = device;
            this.StartX = 0;
            this.StartY = 0;
            this.StartZ = 0;
            this.HueIntervall = 1;
            this.SaturationPercentage = 100;
            this.BrightnessvaluePercentage = 100;

            this.viewTimer = new Timer(40.0);
            this.viewTimer.Elapsed += new ElapsedEventHandler(this.ViewTimer_Elapsed);

            this.hueValue = 0.0;
            this.recentlyUsedColors = new List<Color>();

            for (int i = 0; i < 14; i++)
            {
                this.recentlyUsedColors.Add(Color.Black);
            }
        }

        /// <summary>
        /// Gets or sets the x start coordinate
        /// </summary>
        public int StartX { get; set; }

        /// <summary>
        /// Gets or sets the y start coordinate
        /// </summary>
        public int StartY { get; set; }

        /// <summary>
        /// Gets or sets the z start coordinate
        /// </summary>
        public int StartZ { get; set; }

        /// <summary>
        /// Gets or sets the intervall for hue
        /// </summary>
        public int HueIntervall { get; set; }

        /// <summary>
        /// Gets or sets the saturation percentage
        /// </summary>
        public int SaturationPercentage { get; set; }

        /// <summary>
        /// Gets or sets the brightnessvalue percentage
        /// </summary>
        public int BrightnessvaluePercentage { get; set; }

        /// <summary>
        /// Starts the color gradient
        /// </summary>
        public void Start()
        {
            this.viewTimer.Start();
        }

        /// <summary>
        /// Stops the color gradient
        /// </summary>
        public void Stop()
        {
            this.viewTimer.Stop();
        }

        /// <summary>
        /// Create new image and show it
        /// </summary>
        /// <param name="sender">Sender object</param>
        /// <param name="e">Event arguments</param>
        private void ViewTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            // Get next color and remove last
            this.recentlyUsedColors.Insert(0, this.GetNextColor());
            this.recentlyUsedColors.RemoveAt(this.recentlyUsedColors.Count - 1);

            // Creat image
            List<byte> cubeImageBytes = new List<byte>(LedColorGradient.Capacity);
            for (int x = 0; x < LedColorGradient.AmountOfLedsEachSide; x++)
            {
                for (int y = 0; y < LedColorGradient.AmountOfLedsEachSide; y++)
                {
                    for (int z = 0; z < LedColorGradient.AmountOfLedsEachSide; z++)
                    {
                        // Offset to start coordinate
                        int offset = Math.Abs(this.StartX - x) + Math.Abs(this.StartY - y) + Math.Abs(this.StartZ - z);
                        Color ledcolor = this.recentlyUsedColors[offset];
                        cubeImageBytes.Add(ledcolor.R);
                        cubeImageBytes.Add(ledcolor.G);
                        cubeImageBytes.Add(ledcolor.B);
                    }
                }
            }

            // Send data to device
            this.device.SendCommandToDevice(Convert.ToByte('G'), cubeImageBytes.ToArray(), 10);
        }

        /// <summary>
        /// Generates the new colour for the central of the LED-matrix
        /// </summary>
        /// <returns>Generated colour as int in RGB</returns>
        private Color GetNextColor()
        {
            Color color = this.ConvertHsvToRgb(this.hueValue, this.SaturationPercentage / 100.0, this.BrightnessvaluePercentage / 100.0);
            this.hueValue = (360 + this.hueValue + this.HueIntervall) % 360;
            return color;
        }

        /// <summary>
        /// Converts a hsv-colour to RGB
        /// </summary>
        /// <param name="h">Hue as double</param>
        /// <param name="s">Saturation as double</param>
        /// <param name="v">Value as double</param>
        /// <returns>RGB-Colour as Color-Object</returns>
        private Color ConvertHsvToRgb(double h, double s, double v)
        {
            double r = 0, g = 0, b = 0;

            if (s == 0)
            {
                r = v;
                g = v;
                b = v;
            }
            else
            {
                int i;
                double f, p, q, t;

                if (h == 360)
                {
                    h = 0;
                }
                else
                {
                    h = h / 60;
                }

                i = (int)Math.Truncate(h);
                f = h - i;

                p = v * (1.0 - s);
                q = v * (1.0 - (s * f));
                t = v * (1.0 - (s * (1.0 - f)));

                switch (i)
                {
                    case 0:
                        r = v;
                        g = t;
                        b = p;
                        break;

                    case 1:
                        r = q;
                        g = v;
                        b = p;
                        break;

                    case 2:
                        r = p;
                        g = v;
                        b = t;
                        break;

                    case 3:
                        r = p;
                        g = q;
                        b = v;
                        break;

                    case 4:
                        r = t;
                        g = p;
                        b = v;
                        break;

                    default:
                        r = v;
                        g = p;
                        b = q;
                        break;
                }
            }

            return Color.FromArgb(255, (byte)(r * 255), (byte)(g * 255), (byte)(b * 255));
        }
    }
}