﻿//-----------------------------------------------------------------------------
// <copyright file="LedSnake.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2012 eQ-3 Entwicklung GmbH
// </copyright>
//-----------------------------------------------------------------------------
namespace RgbCubeDemoApp
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.Timers;

    /// <summary>
    /// Wrapper class for the LED-Snake
    /// </summary>
    public class LedSnake
    {
        /// <summary>
        /// Device for the LED-Snake
        /// </summary>
        private ComPortDevice device;

        /// <summary>
        /// Current position of the LED-Snake
        /// </summary>
        private LedCoordinate currentCoordinate;

        /// <summary>
        /// Last positions of the LED-Snake
        /// </summary>
        private Queue<LedCoordinate> coordinateQueue;

        /// <summary>
        /// Timer for showing the next Image
        /// </summary>
        private Timer viewTimer;

        /// <summary>
        /// Current hue value
        /// </summary>
        private double hueValue;

        /// <summary>
        /// Initializes a new instance of the LedSnake class
        /// </summary>
        /// <param name="device">Device for the LED-Snake</param>
        public LedSnake(ComPortDevice device)
        {
            this.device = device;

            this.viewTimer = new Timer(100.0);
            this.viewTimer.Elapsed += new ElapsedEventHandler(this.ViewTimer_Elapsed);

            this.hueValue = 0.0;
            this.currentCoordinate = new LedCoordinate();
            this.coordinateQueue = new Queue<LedCoordinate>();
        }
        
        /// <summary>
        /// Starts the color gradient
        /// </summary>
        public void Start()
        {
            // Turn all leds off
            this.ShowLedSnakeInit();

            this.viewTimer.Start();
        }

        /// <summary>
        /// Stops the color gradient
        /// </summary>
        public void Stop()
        {
            this.viewTimer.Stop();
        }

        /// <summary>
        /// Move snake in the intervall
        /// </summary>
        /// <param name="sender">Sender object</param>
        /// <param name="e">Event arguments</param>
        private void ViewTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            this.ShowLedSnakeMain();
        }        

        /// <summary>
        /// Preparation for the LED-Snake
        /// </summary>
        private void ShowLedSnakeInit()
        {
            // x * y * z * 3 byte
            const int CAPACITY = 5 * 5 * 5 * 3;

            // Colour all LED with #000000 (black)
            List<byte> cubeImageBytes = new List<byte>(CAPACITY);
            for (int i = 0; i < CAPACITY; i++)
            {
                cubeImageBytes.Add(0x00);
            }

            this.device.SendCommandToDevice(Convert.ToByte('G'), cubeImageBytes.ToArray(), 10);
        }

        /// <summary>
        /// Main-method of the LED-Snake: The position gets moved, the new LED gets turned on, the last one gets turned out
        /// </summary>
        private void ShowLedSnakeMain()
        {
            // Get new head of the snake
            this.coordinateQueue.Enqueue(this.currentCoordinate.MovePoint());

            this.ChangeLedAtCoordinate(this.currentCoordinate, true);

            System.Threading.Thread.Sleep(5);

            // Turn old last position off
            if (this.coordinateQueue.Count > 4)
            {
                this.ChangeLedAtCoordinate(this.coordinateQueue.Dequeue(), false);
            }
        }

        /// <summary>
        /// Turns a specific on or off
        /// </summary>
        /// <param name="ledCoordinate">Coordinate of the LED</param>
        /// <param name="wantOn">Specifies the wanted State of the LED (true --> on, false --> off)</param>
        private void ChangeLedAtCoordinate(LedCoordinate ledCoordinate, bool wantOn)
        {
            byte opcode;
            byte[] para;

            if (wantOn)
            {
                Color color = this.GetNextColor();
                opcode = Convert.ToByte('S'); // 'S' --> turn single LED on with color

                para = new byte[6];
                para[0] = (byte)ledCoordinate.X;
                para[1] = (byte)ledCoordinate.Y;
                para[2] = (byte)ledCoordinate.Z;
                para[3] = color.R;
                para[4] = color.G;
                para[5] = color.B;
            }
            else
            {
                opcode = Convert.ToByte('C'); // 'C' --> turn single LED off

                para = new byte[3];
                para[0] = (byte)ledCoordinate.X;
                para[1] = (byte)ledCoordinate.Y;
                para[2] = (byte)ledCoordinate.Z;
            }

            try
            {
                this.device.SendCommandToDevice(opcode, para, 10);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
        }

        /// <summary>
        /// Generates the new colour for the central of the LED-matrix
        /// </summary>
        /// <returns>Generated colour as int in RGB</returns>
        private Color GetNextColor()
        {
            Color color = this.ConvertHsvToRgb(this.hueValue, 1.0, 1.0);
            this.hueValue = (this.hueValue + 3) % 360;
            return color;
        }

        /// <summary>
        /// Converts a hsv-colour to RGB
        /// </summary>
        /// <param name="h">Hue as double</param>
        /// <param name="s">Saturation as double</param>
        /// <param name="v">Value as double</param>
        /// <returns>RGB-Colour as Color-Object</returns>
        private Color ConvertHsvToRgb(double h, double s, double v)
        {
            double r = 0, g = 0, b = 0;

            if (s == 0)
            {
                r = v;
                g = v;
                b = v;
            }
            else
            {
                int i;
                double f, p, q, t;

                if (h == 360)
                {
                    h = 0;
                }
                else
                {
                    h = h / 60;
                }

                i = (int)Math.Truncate(h);
                f = h - i;

                p = v * (1.0 - s);
                q = v * (1.0 - (s * f));
                t = v * (1.0 - (s * (1.0 - f)));

                switch (i)
                {
                    case 0:
                        r = v;
                        g = t;
                        b = p;
                        break;

                    case 1:
                        r = q;
                        g = v;
                        b = p;
                        break;

                    case 2:
                        r = p;
                        g = v;
                        b = t;
                        break;

                    case 3:
                        r = p;
                        g = q;
                        b = v;
                        break;

                    case 4:
                        r = t;
                        g = p;
                        b = v;
                        break;

                    default:
                        r = v;
                        g = p;
                        b = q;
                        break;
                }
            }

            return Color.FromArgb(255, (byte)(r * 255), (byte)(g * 255), (byte)(b * 255));
        }
    }
}
