﻿//-----------------------------------------------------------------------------
// <copyright file="Menu.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2012 eQ-3 Entwicklung GmbH
// </copyright>
//-----------------------------------------------------------------------------
namespace RgbCubeDemoApp
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.IO;
    using System.IO.Ports;

    /// <summary>
    /// Menu of the demo software for the RGB-Cube
    /// </summary>
    public class Menu
    {
        #region Konstanten der Verbindung
        /// <summary>
        /// Baud-Rate (serieller Port)
        /// </summary>
        private const int BAUDRATE = 128000;

        /// <summary>
        /// Daten-Bits (serieller Port)
        /// </summary>
        private const int DATABITS = 8;

        /// <summary>
        /// Parität (serieller Port)
        /// </summary>
        private const Parity PARITY = Parity.None;

        /// <summary>
        /// Stopp-Bits (serieller Port)
        /// </summary>
        private const StopBits STOPBITS = StopBits.One;

        /// <summary>
        /// Lese-Timeout (serieller Port)exit
        /// </summary>
        private const int READTIMEOUT = 1000; // 1 Sekunden
        #endregion

        /// <summary>
        /// Com-Port of the device
        /// </summary>
        private static SerialPort serialPort;

        /// <summary>
        /// Wrapper for the Com-Port
        /// </summary>
        private static ComPortDevice device;

        /// <summary>
        /// Main-method of the console app
        /// </summary>
        /// <param name="args">Parameter for the console app (not needed here)</param>
        public static void Main(string[] args)
        {
            try
            {
                Console.WriteLine("=== Demoanwendung zur Ansteuerung des RGB-Cubes ===");
                Console.WriteLine();

                Menu.ConnectToComPort();
                Menu.ShowMenu();
            }
            finally
            {
                Menu.DisconnectFromComPort();
            }
        }

        /// <summary>
        /// Connect to a Com-Port
        /// </summary>
        private static void ConnectToComPort()
        {
            bool connected = false;

            do
            {
                int numberOfComPort = Menu.GetComPortNumber();
                Menu.serialPort = new SerialPort(
                        string.Format("COM{0}", numberOfComPort),
                        BAUDRATE,
                        PARITY,
                        DATABITS,
                        STOPBITS);
                try
                {
                    Menu.serialPort.ReadTimeout = 500;
                    Menu.serialPort.WriteTimeout = 500;
                    Menu.serialPort.Open();
                    connected = true;

                    Menu.device = new ComPortDevice(serialPort);
                }
                catch (UnauthorizedAccessException)
                {
                    Console.WriteLine(string.Format("FEHLER: Mit COM{0} wurde bereits eine Verbindung hergestellt.", numberOfComPort));
                    connected = false;
                }
                catch (IOException)
                {
                    Console.WriteLine(string.Format("FEHLER: Mit COM{0} ist keine Verbindung möglich.", numberOfComPort));
                    connected = false;
                }
            }
            while (!connected);
        }

        /// <summary>
        /// Get the number of the Com-Port from the user
        /// </summary>
        /// <returns>the number of the Com-Port</returns>
        private static int GetComPortNumber()
        {
            int numberOfComPort = 1;
            bool validInput = false;

            do
            {
                Console.Write("Nummer des ComPorts: ");
                try
                {
                    numberOfComPort = int.Parse(Console.ReadLine());
                    if (numberOfComPort < 0 || numberOfComPort > 100)
                    {
                        throw new Exception();
                    }

                    validInput = true;
                }
                catch
                {
                    Console.WriteLine("FEHLER: Keine gültige ComPort-Nummer");
                }
            }
            while (!validInput);

            return numberOfComPort;
        }

        /// <summary>
        /// Show the menu of the console app
        /// </summary>
        private static void ShowMenu()
        {
            bool appShouldRun = true;

            // Pre-condition for option 1 and 2: Turning on the preview-mode
            Menu.SetPreviewMode();

            do
            {
                Console.WriteLine();
                Console.WriteLine("Beispielanwendungen: ");
                Console.WriteLine("[1] - alle LEDs auf eine Farbe stellen");
                Console.WriteLine("[2] - bewegende LED-Schlange");
                Console.WriteLine("[3] - Farbverlauf");
                Console.WriteLine("[4] - Modus umstellen");
                Console.WriteLine("[5] - LED-Bingo");
                Console.WriteLine("[9] - Anwendung beenden");
                Console.WriteLine();

                int choice = Menu.GetChoice();

                switch (choice)
                {
                    case 1:
                        Menu.TurnOnAllTheLeds();
                        break;
                    case 2:
                        Menu.ShowLedSnake();
                        break;
                    case 3:
                        Menu.ShowColourGradient();
                        break;
                    case 4:
                        Menu.SwitchMode();
                        break;
                    case 5:
                        Menu.ShowLedBingo();
                        break;
                    case 9:
                        appShouldRun = false;
                        break;
                }

                Console.WriteLine();
                Console.WriteLine();
            }
            while (appShouldRun);
        }

        /// <summary>
        /// Get the menu choice from the user
        /// </summary>
        /// <returns>Menu choice</returns>
        private static int GetChoice()
        {
            int choice = 9;
            bool validInput = false;
            List<int> validChoices = new List<int>();
            validChoices.Add(1);
            validChoices.Add(2);
            validChoices.Add(3);
            validChoices.Add(4);
            validChoices.Add(5);
            validChoices.Add(9);

            do
            {
                try
                {
                    Console.Write("Ihre Auswahl: ");
                    choice = int.Parse(Console.ReadLine());

                    if (!validChoices.Contains(choice))
                    {
                        throw new Exception();
                    }

                    validInput = true;
                }
                catch
                {
                    Console.WriteLine("Keine gültige Auswahl");
                }
            }
            while (!validInput);

            Console.WriteLine();
            return choice;
        }

        /// <summary>
        /// Set the preview-mode on the device
        /// </summary>
        private static void SetPreviewMode()
        {
            byte opcodePreviewMode = Convert.ToByte('P');
            byte[] result = device.SendCommandToDevice(opcodePreviewMode, new byte[0], 10);
        }

        /// <summary>
        /// Option 1: Turning on all LEDs of the RGB-Cube in one specific colour
        /// </summary>
        private static void TurnOnAllTheLeds()
        {
            const int CAPACITY = 5 * 5 * 5 * 3; // x * y * z * 3 Bytes je Speicherzelle

            Console.WriteLine("=== Umschalten aller LEDs auf eine Farbe ===");
            Color colour = Color.FromArgb(Menu.GetRgbColor());

            byte[] cubeImageBytes = new byte[CAPACITY];
            for (int x = 0; x < 5; x++)
            {
                for (int y = 0; y < 5; y++)
                {
                    for (int z = 0; z < 5; z++)
                    {
                        int target = (x * 75) + (y * 15) + (z * 3);
                        cubeImageBytes[target] = colour.R;
                        cubeImageBytes[target + 1] = colour.G;
                        cubeImageBytes[target + 2] = colour.B;
                    }
                }
            }

            Menu.device.SendCommandToDevice(Convert.ToByte('G'), cubeImageBytes, 10);
        }

        /// <summary>
        /// Option 2: Show a LED-Snake on the RGB-Cube
        /// </summary>
        private static void ShowLedSnake()
        {
            Console.WriteLine("=== LED-Schlange ===");

            LedSnake ledSnake = new LedSnake(device);
            ledSnake.Start();

            Menu.PressAnyKey();

            ledSnake.Stop();
        }

        /// <summary>
        /// Option 3: Shows a colour gradient on the RGB-Cube
        /// </summary>
        private static void ShowColourGradient()
        {
            Console.WriteLine("=== Farbverlauf ===");
            Console.WriteLine("Startpunkt:");
            int x = Menu.GetValidUInt("X-Koordinate", 4);
            int y = Menu.GetValidUInt("Y-Koordinate", 4);
            int z = Menu.GetValidUInt("Z-Koordinate", 4);
            int hueIntervall = Menu.GetValidInt("Farbtonwechselintervall:", 30);
            int saturation = Menu.GetValidUInt("Farbsättigung in %:", 100);
            int brightnessvalue = Menu.GetValidUInt("Helligkeitswert in %:", 100);
            LedColorGradient ledColorGradient = new LedColorGradient(device);
            ledColorGradient.StartX = x;
            ledColorGradient.StartY = y;
            ledColorGradient.StartZ = z;
            ledColorGradient.HueIntervall = hueIntervall;
            ledColorGradient.SaturationPercentage = saturation;
            ledColorGradient.BrightnessvaluePercentage = brightnessvalue;
            ledColorGradient.Start();

            Menu.PressAnyKey();

            ledColorGradient.Stop();
        }

        /// <summary>
        /// Option 4: Switch the mode of the RGB-Cube
        /// </summary>
        private static void SwitchMode()
        {
            Console.WriteLine("=== Umschalten des Modus des RGB-Cubes ===");

            Console.WriteLine("D --> Demomodus");
            Console.WriteLine("N --> normaler Modus");
            Console.WriteLine("P --> Preview-Modus");
            Console.WriteLine("! --> Bootloader");
            Console.WriteLine();

            char mode = Menu.GetValidMode();

            Menu.device.SendCommandToDevice(Convert.ToByte(mode), new byte[0], 5);
        }

        /// <summary>
        /// Option 5: Shows a random Bingo game
        /// </summary>
        private static void ShowLedBingo()
        {
            Console.WriteLine("=== LED-Bingo ===");

            int totalBingos = Menu.GetTotalBingos();
            int maxLeds = Menu.GetMaxLeds();
            LedBingo ledBingo = new LedBingo(device, totalBingos, maxLeds);
            ledBingo.Start();

            Menu.PressAnyKey();

            ledBingo.Stop();
        }
        
        /// <summary>
        /// Get the rgb-color from the user
        /// </summary>
        /// <returns>Rgb-color as int</returns>
        private static int GetRgbColor()
        {
            int result = -1;
            bool validResult = false;

            do
            {
                Console.WriteLine("RGB-Farbe der LEDs als Hexstring");
                Console.WriteLine("z.B. FF0000 (Rot), 00FF00 (Grün), 0000FF (Blau)");
                Console.Write("RGB-Farbe: ");

                try
                {
                    result = int.Parse(Console.ReadLine().Replace(" ", string.Empty), System.Globalization.NumberStyles.AllowHexSpecifier);

                    if (result >= 0x000000 && result <= 0xFFFFFF)
                    {
                        validResult = true;
                    }
                }
                catch
                {
                    Console.WriteLine("Ihre Eingabe ist keine gültige Farbe.");
                }
            }
            while (!validResult);

            return result;
        }

        /// <summary>
        /// Get a valid mode from the user
        /// </summary>
        /// <returns>Valid mode as char</returns>
        private static char GetValidMode()
        {
            char result = '0';
            bool validInput = false;
            List<char> validModes = new List<char>(new char[] { 'D', 'N', 'P', '!' });

            do
            {
                Console.Write("Modus des RGB-Cubes: ");

                try
                {
                    result = char.Parse(Console.ReadLine());

                    if (validModes.Contains(result))
                    {
                        validInput = true;
                    }
                    else
                    {
                        throw new Exception();
                    }
                }
                catch
                {
                    Console.WriteLine("Ihre Eingabe ist kein gültiger Modus.");
                }
            }
            while (!validInput);

            return result;
        }

        /// <summary>
        /// Gets a valid unsigned int value from user input
        /// </summary>
        /// <param name="type">String that represents the type of input</param>
        /// <param name="max">Maximumm value</param>
        /// <returns>Coordinate value</returns>
        private static int GetValidUInt(string type, int max)
        {
            int result = 0;
            bool validInput = false;
            do
            {
                Console.Write(string.Format("{0} (0 bis {1}):", type, max));

                try
                {
                    result = int.Parse(Console.ReadLine());
                    if (result >= 0 && result <= max)
                    {
                        validInput = true;
                    }
                    else
                    {
                        throw new Exception();
                    }
                }
                catch
                {
                    Console.WriteLine("Ihre Eingabe ist kein gültiger Wert.");
                }
            }
            while (!validInput);
            return result;
        }

        /// <summary>
        /// Gets a valid int value from user input
        /// </summary>
        /// <param name="type">String that represents the type of input</param>
        /// <param name="max">Maximumm value</param>
        /// <returns>Coordinate value</returns>
        private static int GetValidInt(string type, int max)
        {
            int result = 0;
            bool validInput = false;
            do
            {
                Console.Write(string.Format("{0} (-{1} bis {1}):", type, max));

                try
                {
                    result = int.Parse(Console.ReadLine());
                    if (result >= (max * -1) && result <= max)
                    {
                        validInput = true;
                    }
                    else
                    {
                        throw new Exception();
                    }
                }
                catch
                {
                    Console.WriteLine("Ihre Eingabe ist kein gültiger Wert.");
                }
            }
            while (!validInput);
            return result;
        }

        /// <summary>
        /// Get the total amount of Bingos from the user
        /// </summary>
        /// <returns>valid amount of maximal Bingos</returns>
        private static int GetTotalBingos()
        {
            int totalBingos = 0;
            bool validInput = false;
            do
            {
                Console.Write(string.Format("Bitte geben Sie die gewünschte Anzahl der Bingos (1 bis 109) an: "));

                try
                {
                    totalBingos = int.Parse(Console.ReadLine());
                    if (totalBingos >= 1 && totalBingos <= 109)
                    {
                        validInput = true;
                    }
                    else
                    {
                        throw new Exception();
                    }
                }
                catch
                {
                    Console.WriteLine("Ihre Eingabe ist kein gültiger Wert!");
                    Console.WriteLine();
                }
            }
            while (!validInput);
            return totalBingos;
        }

        /// <summary>
        /// Get the maximal amount of LEDs from the user
        /// </summary>
        /// <returns>Valid amount of maximal LEDs.</returns>
        private static int GetMaxLeds()
        {
            int maxLeds = 0;
            bool validInput = false;
            do
            {
                Console.Write(string.Format("Bitte geben Sie die gewünschte Anzahl der LEDs (1 bis 125) an: "));

                try
                {
                    maxLeds = int.Parse(Console.ReadLine());
                    if (maxLeds >= 1 && maxLeds <= 125)
                    {
                        validInput = true;
                    }
                    else
                    {
                        throw new Exception();
                    }
                }
                catch
                {
                    Console.WriteLine("Ihre Eingabe ist kein gültiger Wert!");
                    Console.WriteLine();
                }
            }
            while (!validInput);
            return maxLeds;
        }

        /// <summary>
        /// Acknowledgement to continue
        /// </summary>
        private static void PressAnyKey()
        {
            Console.Write("   bestätigen zum Fortfahren...");
            Console.ReadKey();
        }

        /// <summary>
        /// Close the connection to the Com-Port
        /// </summary>
        private static void DisconnectFromComPort()
        {
            if (Menu.serialPort != null)
            {
                Menu.serialPort.Close();
            }
        }
    }
}
