﻿//-----------------------------------------------------------------------------
// <copyright file="MainWindow.xaml.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2012 eQ-3 Entwicklung GmbH
// </copyright>
//-----------------------------------------------------------------------------
namespace RgbCubeSpectrumAnalyzer
{
    using System;
    using System.IO;
    using System.IO.Ports;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Threading;
    using Microsoft.Win32;
    using System.Collections.ObjectModel;
    using System.Threading;

    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        #region connection constants
        /// <summary>
        /// Baud-Rate (serieller Port)
        /// </summary>
        private const int BAUDRATE = 128000;

        /// <summary>
        /// Daten-Bits (serieller Port)
        /// </summary>
        private const int DATABITS = 8;

        /// <summary>
        /// Parität (serieller Port)
        /// </summary>
        private const Parity PARITY = Parity.None;

        /// <summary>
        /// Stopp-Bits (serieller Port)
        /// </summary>
        private const StopBits STOPBITS = StopBits.One;

        /// <summary>
        /// Lese-Timeout (serieller Port)exit
        /// </summary>
        private const int READTIMEOUT = 1000; // 1 Sekunden
        #endregion

        /// <summary>
        /// current state of playback
        /// </summary>
        private PlaybackState currentPlaybackState;

        private StopAction currentStopAction;

        /// <summary>
        /// RGB-Cube as ComPort-Device
        /// </summary>
        private ComPortDevice device;

        /// <summary>
        /// spectrum analyzer which manages playback of media file and parsing data of stream
        /// </summary>
        private SpectrumAnalyzer spectrumAnalyzer;

        /// <summary>
        /// Wrapper for triggering the RGB-Cube with new data from spectrum analyzer.
        /// </summary>
        private TriggerRgbCubeWithSpectrum triggerRgbCube;

        /// <summary>
        /// Dispatcher-Timer for triggering the RGB-Cube
        /// </summary>
        private DispatcherTimer triggerCubeTimer;

        /// <summary>
        /// Current start point of the shown spectrum
        /// </summary>
        private int startPointOffset;

        /// <summary>
        /// Initializes a new instance of the MainWindow class.
        /// </summary>
        public MainWindow()
        {
            this.InitializeComponent();
            this.TogglePlayerButtonText = MainWindow.PlayerButtonTextPlay;

            this.currentPlaybackState = PlaybackState.Stop;
            this.currentStopAction = StopAction.Next;
            this.ConnectButtonText = MainWindow.ConnectButtonConnect;

            this.AvailableComports = new ObservableCollection<string>();
            foreach (string comport in SerialPort.GetPortNames())
            {
                this.AvailableComports.Add(comport);
            }

            if (this.AvailableComports.Count != 0)
            {
                this.ComPortNumber = this.AvailableComports[0];
            }

            this.Tracklist = new ObservableCollection<string>();

            this.DataContext = this;

            this.spectrumAnalyzer = new SpectrumAnalyzer(
                   TriggerRgbCubeWithSpectrum.DataLength,
                   TriggerRgbCubeWithSpectrum.RelevantData);

            this.spectrumAnalyzer.PlaybackStopped += new EventHandler(this.SpectrumAnalyzer_PlaybackStopped);

            this.triggerCubeTimer = new DispatcherTimer();
            this.triggerCubeTimer.Tick += new EventHandler(this.TriggerCubeTimer_Tick);
            this.triggerCubeTimer.Interval = new TimeSpan(0, 0, 0, 0, 100);
        }

        /// <summary>
        /// enum for current state of playback.
        /// </summary>
        public enum PlaybackState
        {
            /// <summary>
            /// playback is running
            /// </summary>
            Play,

            /// <summary>
            /// playback is paused
            /// </summary>
            Pause,

            /// <summary>
            /// playback is stopped
            /// </summary>
            Stop
        }

        /// <summary>
        /// enum for actions when playback is stopped.
        /// </summary>
        public enum StopAction
        { 
            /// <summary>
            /// no action happens
            /// </summary>
            Stop,

            /// <summary>
            /// playback starts again
            /// </summary>
            Play,

            /// <summary>
            /// skips to next audio track
            /// </summary>
            Next,

            /// <summary>
            /// skips to previous audio track
            /// </summary>
            Previous
        }

        #region release resources
        /// <summary>
        /// OnClosing-event of MainWindow.
        /// </summary>
        /// <param name="e">standard parameter e</param>
        protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
        {
            if (this.spectrumAnalyzer != null)
            {
                this.spectrumAnalyzer.Close();
            }

            if (this.device != null)
            {
                this.device.SerialPort.Close();
            }

            base.OnClosing(e);
        }
        #endregion

        #region manage gui-elements
        /// <summary>
        /// Gets current start point.
        /// </summary>
        /// <param name="sender">standard parameter sender</param>
        /// <param name="e">standard parameter e</param>
        private void SetStartPointComboBox_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            this.startPointOffset = this.cbSelectStartPoint.SelectedIndex;
        }
        #endregion

        #region manage connection to ComPort
        /// <summary>
        /// Toggles the connection to the ComPort.
        /// </summary>
        /// <param name="sender">standard parameter sender</param>
        /// <param name="e">standard parameter e</param>
        private void ToggleComPortConnection_Click(object sender, RoutedEventArgs e)
        {
            if (this.ConnectedToComPort)
            {
                this.DisconnectFromComPort();
            }
            else
            {
                this.ConnectToComPort();
            }
        }

        /// <summary>
        /// Connects to the wished ComPort.
        /// </summary>
        private void ConnectToComPort()
        {
            SerialPort serialPort = new SerialPort(
                this.ComPortNumber,
                BAUDRATE,
                PARITY,
                DATABITS,
                STOPBITS);

            try
            {
                serialPort.ReadTimeout = 500;
                serialPort.WriteTimeout = 500;
                serialPort.Open();
            }
            catch (UnauthorizedAccessException)
            {
                MessageBox.Show(
                    string.Format("FEHLER: Mit {0} wurde bereits eine Verbindung hergestellt.", this.ComPortNumber),
                    "Fehler",
                    MessageBoxButton.OK,
                    MessageBoxImage.Error);
                this.ConnectedToComPort = false;
                return;
            }
            catch (IOException)
            {
                MessageBox.Show(
                    string.Format("FEHLER: Mit {0} ist keine Verbindung möglich.", this.ComPortNumber),
                    "Fehler",
                    MessageBoxButton.OK,
                    MessageBoxImage.Error);
                this.ConnectedToComPort = false;
                return;
            }

            this.device = new ComPortDevice(serialPort);
            this.ConnectedToComPort = true;
            this.ConnectButtonText = MainWindow.ConnectButtonDisconnect;
            this.ComPortConnectionEditable = false;

            this.triggerRgbCube = new TriggerRgbCubeWithSpectrum(this.device);
        }

        /// <summary>
        /// Disconnects from a ComPort.
        /// </summary>
        private void DisconnectFromComPort()
        {
            if (this.currentPlaybackState != PlaybackState.Stop)
            {
                MessageBox.Show(
                    "Bevor die Verbindung zum ComPort beendet werden kann, muss die Wiedergabe gestoppt werden.",
                    "Fehler",
                    MessageBoxButton.OK,
                    MessageBoxImage.Error);
                    
                return;
            }

            this.device.SerialPort.Close();
            this.ConnectedToComPort = false;
            this.ConnectButtonText = MainWindow.ConnectButtonConnect;
            this.ComPortConnectionEditable = true;

            this.triggerRgbCube = null;
        }
        #endregion

        #region manage playback
        /// <summary>
        /// Click-Event of the play-/pause-button.
        /// </summary>
        /// <param name="sender">standard parameter sender</param>
        /// <param name="e">standard parameter e</param>
        private void TogglePlaybackButton_Click(object sender, RoutedEventArgs e)
        {
            switch (this.currentPlaybackState)
            {
                case PlaybackState.Play:
                    this.spectrumAnalyzer.Pause();
                    this.TogglePlayerButtonText = MainWindow.PlayerButtonTextPlay;
                    this.currentPlaybackState = PlaybackState.Pause;
                    break;

                case PlaybackState.Pause:
                    this.spectrumAnalyzer.Play();
                    this.TogglePlayerButtonText = MainWindow.PlayerButtonTextPause;
                    this.currentPlaybackState = PlaybackState.Play;
                    break;

                case PlaybackState.Stop:
                    this.StartPlayback();
                    break;
            }
        }

        /// <summary>
        /// Click-Event of the stop-button.
        /// </summary>
        /// <param name="sender">standard parameter sender</param>
        /// <param name="e">standard parameter e</param>
        private void StopPlaybackButton_Click(object sender, RoutedEventArgs e)
        {
            this.currentStopAction = StopAction.Stop;
            this.StopPlayback();
        }

        /// <summary>
        /// Click-Event of the add-file-to-playlist-button.
        /// </summary>
        /// <param name="sender">standard parameter sender</param>
        /// <param name="e">standard parameter e</param>
        private void AddFileToTracklist_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = "mp3-Datei|*.mp3";
            openFileDialog.Multiselect = true;
            if (openFileDialog.ShowDialog() == true)
            {
                foreach (string filename in openFileDialog.FileNames)
                {
                    this.Tracklist.Add(filename);
                    if (this.Tracklist.Count == 1)
                    {
                        this.CurrentMediaFile = this.Tracklist[0];
                    }
                }
            }
        }

        /// <summary>
        /// Click-Event of the remove-file-from-playlist-button.
        /// </summary>
        /// <param name="sender">standard parameter sender</param>
        /// <param name="e">standard parameter e</param>
        private void RemoveFileFromTracklist_Click(object sender, RoutedEventArgs e)
        {
            this.StopPlayback();
            this.Tracklist.Remove(this.CurrentMediaFile);
            if (this.Tracklist.Count > 0)
            {
                this.CurrentMediaFile = this.Tracklist[0];
            }
            else 
            {
                this.CurrentMediaFile = MainWindow.CurrentMediaFileNoFile;
            }
        }

        /// <summary>
        /// Starts playback of the read media file.
        /// </summary>
        private void StartPlayback()
        {
            if (this.CurrentMediaFile == null || this.CurrentMediaFile == string.Empty)
            {
                return;
            }

            if (this.CurrentMediaFile.Equals(CurrentMediaFileNoFile))
            {
                MessageBox.Show(
                    "Es wurde keine Datei ausgewählt",
                    "Fehler",
                    MessageBoxButton.OK,
                    MessageBoxImage.Error);
                return;
            }

            try
            {
                this.spectrumAnalyzer.OpenFile(this.CurrentMediaFile);
            }
            catch (ArgumentException ex)
            {
                MessageBox.Show(
                    ex.Message,
                    "Fehler",
                    MessageBoxButton.OK,
                    MessageBoxImage.Error);

                return;
            }
            
            this.spectrumAnalyzer.Play();

            this.TotalTrackTime = this.spectrumAnalyzer.GetTotalTime().ToString(@"hh\:mm\:ss");

            this.currentPlaybackState = PlaybackState.Play;
            this.TogglePlayerButtonText = MainWindow.PlayerButtonTextPause;
            this.ComPortConnectionEditable = false;
            this.CurrentMediaFile = this.spectrumAnalyzer.File;
            this.triggerCubeTimer.Start();
        }

        /// <summary>
        /// Actions of SpectrumAnalyzer when the playback is stopped.
        /// </summary>
        /// <param name="sender">standard parameter sender</param>
        /// <param name="e">standard parameter e</param>
        void SpectrumAnalyzer_PlaybackStopped(object sender, EventArgs e)
        {
            this.triggerCubeTimer.Stop();
            this.currentPlaybackState = PlaybackState.Stop;
            this.TogglePlayerButtonText = MainWindow.PlayerButtonTextPlay;

            this.CurrentTrackTime = MainWindow.DefaultTrackTime;
            
            int index = this.Tracklist.IndexOf(this.CurrentMediaFile);
            
            switch (this.currentStopAction)
            {
                // playback stays stopped.
                case StopAction.Stop:
                    this.currentStopAction = StopAction.Next;
                    break;

                // playback starts again and skips automatically to next audio track when current track reaches end.
                case StopAction.Play:
                    this.currentStopAction = StopAction.Next;
                    this.StartPlayback();
                    break;

                // playback skips to next audio track.
                case StopAction.Next:
                    if (this.Tracklist.Count == 1)
                    {
                        this.StartPlayback();
                    }
                    else
                    {
                        index++;
                        index = index % this.Tracklist.Count;
                        this.CurrentMediaFile = this.Tracklist[index];
                        this.currentStopAction = StopAction.Next;
                    }
                    break;

                // playback skips to previous audio track.
                case StopAction.Previous:
                    if (this.Tracklist.Count == 1)
                    {
                        this.StartPlayback();
                    }
                    else
                    {
                        index += this.Tracklist.Count - 1;
                        index = index % this.Tracklist.Count;
                        this.CurrentMediaFile = this.Tracklist[index];
                    }

                    this.currentStopAction = StopAction.Next;
                    break;
            }
        }

        /// <summary>
        /// Stops the playback of the media file.
        /// </summary>
        private void StopPlayback()
        {
            this.spectrumAnalyzer.Stop();
        }
        #endregion

        #region manage parsing of spectrum analyzer data and triggerung rgb-cube

        /// <summary>
        /// Tick-method of the Dispatcher-Timer which triggeres the RGB-Cube.
        /// </summary>
        /// <param name="sender">standard parameter sender</param>
        /// <param name="e">standard parameter e</param>
        private void TriggerCubeTimer_Tick(object sender, EventArgs e)
        {
            if (this.currentPlaybackState != PlaybackState.Play || this.triggerRgbCube == null)
            {
                return;
            }

            this.triggerRgbCube.StartPointOffsetForSpectrum = this.startPointOffset;
            this.triggerRgbCube.Trigger(this.spectrumAnalyzer.SpectrumData);
            this.CurrentTrackTime = this.spectrumAnalyzer.GetCurrentTime().ToString(@"hh\:mm\:ss");
        }
        #endregion

        /// <summary>
        /// Plays selected audio track from the playlist.
        /// </summary>
        /// <param name="sender">standard parameter sender</param>
        /// <param name="e">standard parameter e</param>
        private void ListViewTracklist_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (this.triggerRgbCube != null)
            {
                if (this.currentPlaybackState != PlaybackState.Stop)
                {
                    this.currentStopAction = StopAction.Play;
                    this.StopPlayback();
                }
                else
                {
                    this.currentStopAction = StopAction.Next;
                    this.StartPlayback();
                }
            }
        }

        /// <summary>
        /// Click-Event of the skip-to-previous-track-button.
        /// </summary>
        /// <param name="sender">standard parameter sender</param>
        /// <param name="e">standard parameter e</param>
        private void SkipToPreviousTrackButton_Click(object sender, RoutedEventArgs e)
        {
            if (this.triggerRgbCube != null)
            {
                if (this.currentPlaybackState != PlaybackState.Stop)
                {
                    this.currentStopAction = StopAction.Previous;
                    this.StopPlayback();
                }
                else
                {
                    int index = this.Tracklist.IndexOf(this.CurrentMediaFile);
                    index += this.Tracklist.Count - 1;
                    index = index % this.Tracklist.Count;
                    this.CurrentMediaFile = this.Tracklist[index];
                }
            }
        }

        /// <summary>
        /// Click-Event of the skip-to-next-track-button.
        /// </summary>
        /// <param name="sender">standard parameter sender</param>
        /// <param name="e">standard parameter e</param>
        private void SkipToNextTrackButton_Click(object sender, RoutedEventArgs e)
        {
            if (this.triggerRgbCube != null)
            {
                if (this.currentPlaybackState != PlaybackState.Stop)
                {
                    this.currentStopAction = StopAction.Next;
                    this.StopPlayback();
                }
                else
                {
                    int index = this.Tracklist.IndexOf(this.CurrentMediaFile);
                    index++;
                    index = index % this.Tracklist.Count;
                    this.CurrentMediaFile = this.Tracklist[index];
                }
            }
        }
    }
}