﻿//-----------------------------------------------------------------------------
// <copyright file="LedPillar.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2012 eQ-3 Entwicklung GmbH
// </copyright>
//-----------------------------------------------------------------------------
namespace RgbCubeSpectrumAnalyzer
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Windows.Media;

    /// <summary>
    /// Represents a single pillar of the RGB-Cube (const x, const y, variable y).
    /// </summary>
    public class LedPillar
    {
        /// <summary>
        /// Maximal height of a single pillar.
        /// </summary>
        public const int MaxHeightOfSinglePillar = 50;

        /// <summary>
        /// Length of a single face of the RGB-Cube.
        /// </summary>
        public const int LengthOfSingleFace = 5;

        /// <summary>
        /// Height of a single step/ led in the pillar.
        /// </summary>
        public const int HeightOfSingleStep = LedPillar.MaxHeightOfSinglePillar / LedPillar.LengthOfSingleFace;

        /// <summary>
        /// Length of cube image bytes which are sent to the RGB-Cube.
        /// </summary>
        public const int LengthOfCubeImageBytes = LedPillar.LengthOfSingleFace * LedPillar.LengthOfSingleFace * LedPillar.LengthOfSingleFace * 3;
        
        /// <summary>
        /// Initializes a new instance of the LedPillar class.
        /// </summary>
        public LedPillar()
        {
            this.Height = 0;
        }

        /// <summary>
        /// Gets or sets the height of the pillar.
        /// </summary>
        public int Height { get; set; }

        /// <summary>
        /// Gets the leds with colour information of the pillar.
        /// </summary>
        /// <returns>leds with colour information</returns>
        public List<Color> GetLedCoordinates()
        {
            List<Color> ledCoordinates = new List<Color>();
            for (int i = 0; i < LedPillar.LengthOfSingleFace; i++)
            {
                Color colour;
                switch (i)
                {
                    case 0:
                        colour = Color.FromRgb(0x00, 0xFF, 0x00);
                        break;
                    case 1:
                        colour = Color.FromRgb(0xC0, 0xFF, 0x40);
                        break;
                    case 2:
                        colour = Color.FromRgb(0xFF, 0xFF, 0x00);
                        break;
                    case 3:
                        colour = Color.FromRgb(0xFF, 0xA5, 0x00);
                        break;
                    case 4:
                        colour = Color.FromRgb(0xFF, 0x00, 0x00);
                        break;
                    default:
                        colour = Colors.Black;
                        break;
                }                

                if ((i + 1) * LedPillar.HeightOfSingleStep <= this.Height)
                {
                    ledCoordinates.Add(colour);
                }
                else
                {
                    int remainingHeigth = this.Height % LedPillar.HeightOfSingleStep;
                    if (remainingHeigth != 0 && i * LedPillar.HeightOfSingleStep <= this.Height)
                    {
                        colour.R = (byte)Math.Round((double)colour.R * remainingHeigth / LedPillar.HeightOfSingleStep);
                        colour.G = (byte)Math.Round((double)colour.G * remainingHeigth / LedPillar.HeightOfSingleStep);
                        colour.B = (byte)Math.Round((double)colour.B * remainingHeigth / LedPillar.HeightOfSingleStep);

                        ledCoordinates.Add(colour);
                    }
                    else
                    {                        
                        ledCoordinates.Add(Colors.Black);
                    }
                }
            }

            return ledCoordinates;
        }
    }
}