﻿//-----------------------------------------------------------------------------
// <copyright file="TriggerRgbCubeWithSpectrum.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2012 eQ-3 Entwicklung GmbH
// </copyright>
//-----------------------------------------------------------------------------
namespace RgbCubeSpectrumAnalyzer
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Windows.Media;

    /// <summary>
    /// Wrapper for triggering the RGB-Cube with data from the spectrum analyzer.
    /// </summary>
    public class TriggerRgbCubeWithSpectrum
    {
        /// <summary>
        /// The spectrum analyzer returns an array of 256 ints.
        /// This constant determines how many of them are relevant for the RGB-Cube.
        /// </summary>
        public const int RelevantData = 128;

        /// <summary>
        /// Length of information used for a single pillar.
        /// </summary>
        public const int DataLength = 8;

        /// <summary>
        /// Amount of pillars.
        /// </summary>
        public const int AmountOfPillars = TriggerRgbCubeWithSpectrum.RelevantData / TriggerRgbCubeWithSpectrum.DataLength;

        /// <summary>
        /// RGB-Cube as ComPort-Device.
        /// </summary>
        private ComPortDevice device;

        /// <summary>
        /// X*Z-matrix of led-pillars.
        /// </summary>
        private LedPillar[,] ledPillars;

        /// <summary>
        /// Start point of the spectrum from the GUI.
        /// </summary>
        private int startPointOffset;

        /// <summary>
        /// Lock-object for the start point.
        /// </summary>
        private object lockObjectStartPoint = new object();

        /// <summary>
        /// Initializes a new instance of the TriggerRgbCubeWithSpectrum class.
        /// </summary>
        /// <param name="device">RGB-Cube as ComPort-Device.</param>
        public TriggerRgbCubeWithSpectrum(ComPortDevice device)
        {
            if (TriggerRgbCubeWithSpectrum.RelevantData % TriggerRgbCubeWithSpectrum.DataLength != 0)
            {
                throw new InvalidOperationException("Die Anzahl der relevanten Daten muss durch die Datenlänge für eine einzelne Säule absolut teilbar sein");
            }

            this.device = device;
            this.ledPillars = new LedPillar[LedPillar.LengthOfSingleFace, LedPillar.LengthOfSingleFace];
            for (int x = 0; x < LedPillar.LengthOfSingleFace; x++)
            {
                for (int z = 0; z < LedPillar.LengthOfSingleFace; z++)
                {
                    this.ledPillars[x, z] = new LedPillar();
                }
            }

            this.SetRgbCubeInPreviewMode();
        }

        /// <summary>
        /// Gets or sets the start point from the GUI.
        /// </summary>
        public int StartPointOffsetForSpectrum
        {
            get
            {
                lock (this.lockObjectStartPoint)
                {
                    return this.startPointOffset;
                }
            }

            set
            {
                lock (this.lockObjectStartPoint)
                {
                    this.startPointOffset = value;
                }
            }
        }

        /// <summary>
        /// Triggers the RGB-Cube with the data of the spectrum analyzer.
        /// </summary>
        /// <param name="spectrumData">data from the spectrum analyzer</param>
        public void Trigger(List<int> spectrumData)
        {
            List<int> data = this.AdjustData(spectrumData);
            data = this.NormalizeData(data);

            this.SetLedPillars(data);
            this.FillMiddlePillars(data);

            this.ShowPillarsOnCube();
        }

        /// <summary>
        /// Sets the preview mode on connected RGB-Cube.
        /// </summary>
        private void SetRgbCubeInPreviewMode()
        {
            const byte SetPreviewModeCommand = (byte)'P';
            this.device.SendCommandToDevice(SetPreviewModeCommand, new byte[] { }, 0);
        }

        /// <summary>
        /// Adjusts data from spectrum analyzer in regard of data length for a single pillar and the amount of relevant data.
        /// </summary>
        /// <param name="rowData">data to adjust</param>
        /// <returns>adjusted data</returns>
        private List<int> AdjustData(List<int> rowData)
        {
            List<int> adjustedData = new List<int>();
            for (int i = 0; i < TriggerRgbCubeWithSpectrum.RelevantData; i += TriggerRgbCubeWithSpectrum.DataLength)
            {
                List<int> temp = new List<int>();
                temp.AddRange(rowData.GetRange(i, TriggerRgbCubeWithSpectrum.DataLength));

                int tempSum = 0;
                foreach (int number in temp)
                {
                    tempSum += number;
                }

                adjustedData.Add((int)Math.Round((double) tempSum / TriggerRgbCubeWithSpectrum.DataLength));
            }

            return adjustedData;
        }

        /// <summary>
        /// Normalizes data from spectrum analyzer.
        /// </summary>
        /// <param name="adjustedData">adjusted data</param>
        /// <returns>normalized data</returns>
        private List<int> NormalizeData(List<int> adjustedData)
        {
            for (int i = 0; i < TriggerRgbCubeWithSpectrum.AmountOfPillars; i++)
            {
                double factor = LedPillar.MaxHeightOfSinglePillar / (LedPillar.MaxHeightOfSinglePillar - ((LedPillar.MaxHeightOfSinglePillar * ((double)i / TriggerRgbCubeWithSpectrum.AmountOfPillars)) / 2));
                adjustedData[i] = (int)Math.Round(adjustedData[i] * factor);
            }

            return adjustedData;
        }

        /// <summary>
        /// Sets the height of led pillars according to the data of the spectrum analyzer. 
        /// </summary>
        /// <param name="spectrumData">adjusted and normalizes data</param>
        private void SetLedPillars(List<int> spectrumData)
        {
            int counter = spectrumData.Count - this.startPointOffset;
            
            this.ledPillars[0, 0].Height = spectrumData[(counter++) % spectrumData.Count];
            this.ledPillars[1, 0].Height = spectrumData[(counter++) % spectrumData.Count];
            this.ledPillars[2, 0].Height = spectrumData[(counter++) % spectrumData.Count];
            this.ledPillars[3, 0].Height = spectrumData[(counter++) % spectrumData.Count];
            this.ledPillars[4, 0].Height = spectrumData[(counter++) % spectrumData.Count];
            this.ledPillars[4, 1].Height = spectrumData[(counter++) % spectrumData.Count];
            this.ledPillars[4, 2].Height = spectrumData[(counter++) % spectrumData.Count];
            this.ledPillars[4, 3].Height = spectrumData[(counter++) % spectrumData.Count];
            this.ledPillars[4, 4].Height = spectrumData[(counter++) % spectrumData.Count];
            this.ledPillars[3, 4].Height = spectrumData[(counter++) % spectrumData.Count];
            this.ledPillars[2, 4].Height = spectrumData[(counter++) % spectrumData.Count];
            this.ledPillars[1, 4].Height = spectrumData[(counter++) % spectrumData.Count];
            this.ledPillars[0, 4].Height = spectrumData[(counter++) % spectrumData.Count];
            this.ledPillars[0, 3].Height = spectrumData[(counter++) % spectrumData.Count];
            this.ledPillars[0, 2].Height = spectrumData[(counter++) % spectrumData.Count];
            this.ledPillars[0, 1].Height = spectrumData[(counter++) % spectrumData.Count];       
        }

        /// <summary>
        /// Fills the middle pillars with average data of the adjoining pillars.
        /// </summary>
        private void FillMiddlePillars(List<int> spectrumData)
        {
            int temp = 0;

            for(int i = 0; i < spectrumData.Count; i++)
            {
                temp += spectrumData[i];
            }

            temp /= spectrumData.Count;
            this.ledPillars[2, 2].Height = temp;
        }

        /// <summary>
        /// Analyzes the matrix of pillars and sends cube image bytes to the RGB-Cube.
        /// </summary>
        private void ShowPillarsOnCube()
        {
            const byte SetAllLedsCommand = (byte)'G';

            byte[] cubeImageBytes = new byte[LedPillar.LengthOfCubeImageBytes];
            for (int x = 0; x < LedPillar.LengthOfSingleFace; x++)
            {
                for (int z = 0; z < LedPillar.LengthOfSingleFace; z++)
                {
                    List<Color> ledCoordinates = this.ledPillars[x, z].GetLedCoordinates();
                    for (int y = 0; y < LedPillar.LengthOfSingleFace; y++)
                    {
                        int target = (x * 75) + (y * 15) + (z * 3);
                        cubeImageBytes[target] = ledCoordinates[y].R;
                        cubeImageBytes[target + 1] = ledCoordinates[y].G;
                        cubeImageBytes[target + 2] = ledCoordinates[y].B;
                    }
                }
            }

            this.device.SendCommandToDevice(SetAllLedsCommand, cubeImageBytes, 0);
        }
    }
}