﻿namespace RgbCubeSpectrumAnalyzer
{
    using System;
    using System.Windows;
    using NAudio.Wave;
    using NAudio.Wave.SampleProviders;

    public class AudioPlayback : IDisposable
    {
        /// <summary>
        /// Player from the NAudio-library.
        /// </summary>
        private IWavePlayer playbackDevice;

        /// <summary>
        /// Stream of the audio file.
        /// </summary>
        private WaveStream fileStream;

        /// <summary>
        /// Instance of the SampleAggregator.
        /// </summary>
        private readonly SampleAggregator aggregator;

        /// <summary>
        /// Adds or removes an event handler for the calculated data.
        /// </summary>
        public event EventHandler<FftEventArgs> FftCalculated
        {
            add { aggregator.FftCalculated += value; }
            remove { aggregator.FftCalculated -= value; }
        }

        /// <summary>
        /// Adds or removes an event handler for the maximum.
        /// </summary>
        public event EventHandler<MaxSampleEventArgs> MaximumCalculated
        {
            add { aggregator.MaximumCalculated += value; }
            remove { aggregator.MaximumCalculated -= value; }
        }

        /// <summary>
        /// Adds or removes an event handler for the stopped playback.
        /// </summary>
        public event EventHandler PlaybackStopped
        {
            add { this.playbackDevice.PlaybackStopped += value; }
            remove { this.playbackDevice.PlaybackStopped -= value; }
        }

        /// <summary>
        /// Initializes a new instance of the AudioPlayback class
        /// </summary>
        public AudioPlayback()
        {
            this.aggregator = new SampleAggregator();
            this.aggregator.NotificationCount = 882;
            this.aggregator.PerformFFT = true;
            this.playbackDevice = new WaveOut { DesiredLatency = 200 };
        }

        /// <summary>
        /// Loads the audio file.
        /// </summary>
        /// <param name="fileName">Name of the audio file</param>
        public void Load(string fileName)
        {
            this.Stop();
            this.CloseFile();
            this.OpenFile(fileName);
        }

        /// <summary>
        /// Closes the file stream.
        /// </summary>
        private void CloseFile()
        {
            if (this.fileStream != null)
            {
                this.fileStream.Dispose();
                this.fileStream = null;
            }
        }

        /// <summary>
        /// Opens the file stream.
        /// </summary>
        /// <param name="fileName">Name of the audio file</param>
        private void OpenFile(string fileName)
        {
            try
            {
                var inputStream = this.CreateInputStream(fileName);
                playbackDevice.Init(new SampleToWaveProvider(inputStream));
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "Problem opening file");
                this.CloseFile();
            }
        }

        /// <summary>
        /// Creates the input stream.
        /// </summary>
        /// <param name="fileName">Name of the audio file</param>
        /// <returns>The input stream</returns>
        private ISampleProvider CreateInputStream(string fileName)
        {
            if (fileName.EndsWith(".wav"))
            {
                this.fileStream = AudioPlayback.OpenWavStream(fileName);
            }
            else if (fileName.EndsWith(".mp3"))
            {
                this.fileStream = new Mp3FileReader(fileName);
            }
            else
            {
                throw new InvalidOperationException("Unsupported extension");
            }

            var inputStream = new SampleChannel(this.fileStream);
            var sampleStream = new NotifyingSampleProvider(inputStream);
            sampleStream.Sample += (s, e) => this.aggregator.Add(e.Left);
            return sampleStream;
        }
        /// <summary>
        /// Opens the wave stream.
        /// </summary>
        /// <param name="fileName">Name of the audio file</param>
        /// <returns>The wave stream</returns>
        private static WaveStream OpenWavStream(string fileName)
        {
            WaveStream readerStream = new WaveFileReader(fileName);
            if (readerStream.WaveFormat.Encoding != WaveFormatEncoding.Pcm)
            {
                readerStream = WaveFormatConversionStream.CreatePcmStream(readerStream);
                readerStream = new BlockAlignReductionStream(readerStream);
            }

            return readerStream;
        }

        /// <summary>
        /// Starts the playback.
        /// </summary>
        public void Play()
        {
            if (this.playbackDevice != null && this.fileStream != null && this.playbackDevice.PlaybackState != PlaybackState.Playing)
            {
                this.playbackDevice.Play();
            }
        }

        /// <summary>
        /// Pauses the playback.
        /// </summary>
        public void Pause()
        {
            if (this.playbackDevice != null)
            {
                this.playbackDevice.Pause();
            }
        }

        /// <summary>
        /// Stops the playback.
        /// </summary>
        public void Stop()
        {
            if (this.playbackDevice != null)
            {
                this.playbackDevice.Stop();
            }

            if (this.fileStream != null)
            {
                this.fileStream.Position = 0;
            }
        }

        /// <summary>
        /// Gets the current track time.
        /// </summary>
        /// <returns></returns>
        public TimeSpan GetCurrentTime()
        {
            if (this.fileStream != null)
            {
                return this.fileStream.CurrentTime;
            }
            else 
            {
                return new TimeSpan(0);
            }
        }

        /// <summary>
        /// Gets the total track time.
        /// </summary>
        /// <returns></returns>
        public TimeSpan GetTotalTime()
        {
            if (this.fileStream != null)
            {
                return this.fileStream.TotalTime;
            }
            else
            {
                return new TimeSpan(0);
            }
        }

        /// <summary>
        /// Empties the playback device
        /// </summary>
        public void Dispose()
        {
            Stop();
            CloseFile();
            if (this.playbackDevice != null)
            {
                this.playbackDevice.Dispose();
                this.playbackDevice = null;
            }
        }
    }
}