﻿//-----------------------------------------------------------------------------
// <copyright file="SpectrumAnalyzer.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2012 eQ-3 Entwicklung GmbH
// </copyright>
//-----------------------------------------------------------------------------
namespace RgbCubeSpectrumAnalyzer
{
    using System;
    using System.Collections.Generic;
    using System.IO;

    /// <summary>
    /// Represents a spectrum analyzer with playback-options.
    /// </summary>
    public class SpectrumAnalyzer
    {
        public event EventHandler PlaybackStopped
        {
            add { this.audioPlayback.PlaybackStopped += value; }
            remove { this.audioPlayback.PlaybackStopped -= value; }
        }

        /// <summary>
        /// Wrapper for audio playback.
        /// </summary>
        private AudioPlayback audioPlayback;

        /// <summary>
        /// Current data of the spectrum analyzer.
        /// It contains 256 ints: The amount represents the different frequencies and and the number itself represents height of the frequence.
        /// </summary>
        private List<int> spectrumData;

        /// <summary>
        /// The spectrum analyzer returns an array of 256 ints.
        /// This constant determines how many of them are relevant for the RGB-Cube.
        /// </summary>
        private int relevantData;

        /// <summary>
        /// Length of information used for a single pillar.
        /// </summary>
        private int dataLength;

        /// <summary>
        /// Lock-object of the spectrum data.
        /// </summary>
        private object lockSpectrumData = new object();

        /// <summary>
        /// Initializes a new instance of the SpectrumAnalyzer class.
        /// </summary>
        /// <param name="dataLength">data length (compare field docu)</param>
        /// <param name="relevantData">relevant data (compare field docu)</param>
        public SpectrumAnalyzer(int dataLength, int relevantData)
        {
            
            if (relevantData % dataLength != 0)
            {
                throw new ArgumentException("Die Anzahl relevanter Daten muss durch die Datenlänge teilbar sein.");
            }

            this.dataLength = dataLength;
            this.relevantData = relevantData;

            this.audioPlayback = new AudioPlayback();
            this.audioPlayback.FftCalculated += new EventHandler<FftEventArgs>(this.AudioGraph_FftCalculated); 
        }

        /// <summary>
        /// Opens and loads the media file.
        /// Verifies for the right data type.
        /// </summary>
        /// <param name="file">media file</param>
        public void OpenFile(string file)
        {
            if (string.IsNullOrEmpty(file))
            {
                throw new ArgumentException("Ohne mp3-Datei ist kein Abspielen möglich.");
            }

            FileInfo fileInfo = new FileInfo(file);
            if (!fileInfo.Exists)
            {
                throw new ArgumentException(string.Format(
                    "Die angegebene Datei existiert nicht:\n{0}",
                    file));
            }
            else if (fileInfo.Extension != ".mp3")
            {
                throw new ArgumentException(string.Format(
                    "Die angegebene Datei ist keine mp3-Datei:\n{0}",
                    file));
            }

            this.File = file;

            this.audioPlayback.Load(file);
        }

        /// <summary>
        /// Finalizes an instance of the SpectrumAnalyzer class.
        /// Ensures that the audio streams gets closed securely.
        /// </summary>
        ~SpectrumAnalyzer()
        {
           this.audioPlayback.Dispose();
        }

        /// <summary>
        /// Gets the media file.
        /// </summary>
        public string File { get; private set; }

        /// <summary>
        /// Gets or sets securely the data of the spectrum analyzer.
        /// </summary>
        public List<int> SpectrumData
        {
            get
            {
                lock (this.lockSpectrumData)
                {
                    return this.spectrumData;
                }
            }

            set
            {
                lock (this.lockSpectrumData)
                {
                    this.spectrumData = value;
                }
            }
        }

        /// <summary>
        /// Plays current media file.
        /// </summary>
        public void Play()
        {
            this.audioPlayback.Play();
        }

        /// <summary>
        /// Pauses playback of current media file.
        /// </summary>
        public void Pause()
        {
            this.audioPlayback.Pause();
        }

        /// <summary>
        /// Stops playback of current media file.
        /// </summary>
        public void Stop()
        {
            this.audioPlayback.Stop();
        }

        /// <summary>
        /// Closes the audio stream securely.
        /// </summary>
        public void Close()
        {
            this.audioPlayback.Dispose();
        }
        
        /// <summary>
        /// Gets the current played time of the audio track.
        /// </summary>
        /// <returns></returns>
        public TimeSpan GetCurrentTime()
        {
            if (this.audioPlayback != null)
            {
                return this.audioPlayback.GetCurrentTime();
            }
            else
            {
                return new TimeSpan(0);
            }
        }

        /// <summary>
        /// Gets the total runtime of the audio track.
        /// </summary>
        /// <returns></returns>
        public TimeSpan GetTotalTime()
        {
            if (this.audioPlayback != null)
            {
                return this.audioPlayback.GetTotalTime();
            }
            else
            {
                return new TimeSpan(0);
            }
        }

        /// <summary>
        /// Managing new calculated data of the spectrum analyzer.
        /// </summary>
        /// <param name="sender">standard parameter sender</param>
        /// <param name="e">standard parameter e</param>
        private void AudioGraph_FftCalculated(object sender, FftEventArgs e)
        {
            this.ParseCalculatedFft(e.Result);
        }

        /// <summary>
        /// Converts an array of NAudio.Dsp.Complex to a list of int32 (min: 0 - max: 100)
        /// </summary>
        /// <param name="complex">row data of spectrum analyzer</param>
        private void ParseCalculatedFft(NAudio.Dsp.Complex[] complex)
        {
            const int BinsPerPoint = 2;

            List<int> spectrumData = new List<int>();
            for (int n = 0; n < complex.Length / 2; n += BinsPerPoint)
            {
                int position = this.GetSinglePosition(complex[n]);
                spectrumData.Add(Math.Abs(position - 100));
            }

            this.spectrumData = spectrumData;
        }

        /// <summary>
        /// Filters position of a NAudio.Dsp.Complex.
        /// </summary>
        /// <param name="c">NAudio.Dsp.Complex to convert.</param>
        /// <returns>filtered position</returns>
        private int GetSinglePosition(NAudio.Dsp.Complex c)
        {
            double intensityDB = 10 * Math.Log(Math.Sqrt((c.X * c.X) + (c.Y * c.Y)));
            double minDB = -96;
            if (intensityDB < minDB)
            {
                intensityDB = minDB;
            }

            double result = intensityDB / minDB;

            int resultI = (int) Math.Round(result * 100);
            return resultI;
        }

        /// <summary>
        /// Filters data based on transmitted parameters in the constructor.
        /// </summary>
        /// <param name="spectrum">row data of the spectrum analyzer</param>
        /// <returns>result of the action</returns>
        private List<int> FilterSpectrum(List<int> spectrum)
        {
            List<int> result = new List<int>();
            for (int i = 0; i < this.relevantData; i += this.dataLength)
            {
                List<int> temp = spectrum.GetRange(i, this.dataLength);
                result.Add(this.CalculateAverage(temp));
            }

            return result;
        }

        /// <summary>
        /// Calculates average of a list of int32.
        /// </summary>
        /// <param name="list">list of int32</param>
        /// <returns>calculated average</returns>
        private int CalculateAverage(List<int> list)
        {
            int temp = 0;
            foreach (int i in list)
            {
                temp += i;
            }

            return (int)Math.Round((double)temp / list.Count);
        }
    }
}
